// USBsocket.h: interface for the CUSBsocket class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_USBSOCKET_H__8B9CBE16_F075_4636_840E_8C00E4798110__INCLUDED_)
#define AFX_USBSOCKET_H__8B9CBE16_F075_4636_840E_8C00E4798110__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "FTD2XX.h"

#define MAX_MSG 8192+2
#define QUEUE_SIZE 128
#define QUEUE_MASK 0x07F

#define EEROM_SIZE 4


//Socket status
#define SOCKET_NOT_RUNNING 0
#define SOCKET_WAITINGUSB 1
#define SOCKET_WAITINGCONNECT 2
#define SOCKET_CONNECTED 3


typedef struct
{
	int length;
	char buf[MAX_MSG];
} MSG_STRUC;

//////////////////////////////////////////////////////////////////////////////
// highly non-portable ASCP message buffer structure but....its fast and easy
//////////////////////////////////////////////////////////////////////////////
typedef union
{
	struct bts
	{
		unsigned __int8 b0;
		unsigned __int8 b1;
		unsigned __int8 b2;
		unsigned __int8 b3;
		unsigned __int8 parms[8192-4];
	}bytes;
	struct dbts
	{
		unsigned __int8 b0;
		unsigned __int8 b1;
		unsigned __int8 parms[8192-2];
	}dibytes;
	struct dwrds
	{
		unsigned __int16 header;
		unsigned __int16 parms[(8192-2)/2];
	}diwords;
	struct wrds
	{
		unsigned __int16 header;
		unsigned __int16 CItem;
		unsigned __int16 parms[(8192-4)/2];
	}words;
	struct dlngs
	{
		unsigned __int16 header;
		unsigned __int16 parms[(8192-2)/2];
	}data;
	struct lngs
	{
		unsigned __int32 header;
		unsigned __int32 parms[(8192-4)/4];
	}longs;
	unsigned __int8 buf[8194];
}ASCPstruct;

typedef union
{
	struct bts
	{
		unsigned __int8 h0;
		unsigned __int8 h1;
		unsigned __int8 seql;
		unsigned __int8 seqh;
	}bytes;
	unsigned __int8 buf[8196];
}tASCPDatastruct;

typedef union			//highly non portable way to access bytes of short int
{
	struct b4
	{
		unsigned __int8 b0;
		unsigned __int8 b1;
		unsigned __int8 b2;
		unsigned __int8 b3;
	}bytes;
	unsigned int all;
}t32BITS;


class CUSBsocket  
{
public:
	CString GetSocketstatus();
	CString GetUSBstatus(BOOL &Connected);
	CString GetIPs( int & IP);
	CUSBsocket();
	virtual ~CUSBsocket();
	void StartControl(u_short PortNum, CString  SNFilterStr);
	void StopControl();
	void USBThread();
	void FifoThread();
	void SocketThread();
	BOOL IsClientConnected(){return(m_SocketSstatus == SOCKET_CONNECTED);}

	CString m_SdrName;
	CString m_SdrSN;
	unsigned short m_BootVer;
	unsigned short m_FirmVer;
	long m_CurrentIPAddress;

	// Externally exposed variables
private:
	void StopSDRxx();
	static UINT USBThreadProc(LPVOID pParam);	//must be static since is thread
	static UINT SocketThreadProc(LPVOID pParam);
	static UINT USBFifoThreadProc(LPVOID pParam);
	void SendDataToUSB(  char* pData, int length);
	void SendTCPMsgToSocket( ASCPstruct* pASCPmsg, int length);
	void SendUDPMsgToSocket( ASCPstruct* pASCPmsg, int length);
	void SendASCPMsgToUSB( ASCPstruct* pASCPmsg);
	void InterceptSdrMsgs( ASCPstruct* pASCPmsg, int length );
	void SendUDPPacketToSocket( unsigned __int8* pData);
	void SetupFDSets(fd_set& ReadFDs, fd_set& ExceptFDs);
	void AssembleASCPMsg( char* pData, int length );
	void ReadIOMemory( int adr, unsigned int* pData);
	void WriteIOMemory( int adr, unsigned int Data);
	void SetSDRIQSampleRateFromEEROM(unsigned int rate);
	void RequestDeviceInfo();
	

	BOOL OpenUSBdll();
	BOOL ShutdownConnection(SOCKET& sd);
	// local variables
	BOOL m_USBThreadRunning;
	BOOL m_SocketThreadRunning;
	BOOL m_FIFOThreadRunning;
	BOOL m_USBPortOpen;
	BOOL m_USBReInit;
	BOOL m_FiFoReset;
	u_short m_PortNum;
	CString	m_SNFilterStr;

	CWinThread *m_pSocketThread;
	CWinThread *m_pUSBThread;
	CWinThread *m_pUSBFifoThread;
	unsigned short m_SeqNumber;
	int m_RxMsgLength;
	int m_RxMsgIndex;
	int m_MsgState;
	int m_TCPMsgLength;
	int m_TCPMsgIndex;
	int m_TCPMsgState;
	int m_UsbSstatus;
	int m_SocketSstatus;
	int m_Qhead;
	int m_Qtail;
	int m_FifoHead;
	int m_FifoTail;
	int m_KeepAlive;
	unsigned char m_EEROMData[EEROM_SIZE];
	unsigned __int8* m_pUSBBuffer;
	unsigned __int8* m_pUSBFifo;
	CString m_Str;
	FT_STATUS m_FTStatus;
	FT_HANDLE m_FTHandle;
	HMODULE m_hmodule;
	HANDLE m_hReadEvent;
	HANDLE m_hFifoWriteEvent;
	ASCPstruct m_rxASCPmsg;
	ASCPstruct m_TxASCPmsg;
	ASCPstruct m_TCPASCPmsg;
    sockaddr_in m_ClientTCPAddress;
    sockaddr_in m_ClientUDPAddress;
	SOCKET m_ListenSocket;
	SOCKET m_ClientSocket;
	SOCKET m_UDPSocket;
	MSG_STRUC* m_pMsgQue;
	CRITICAL_SECTION m_SockCriticalSection;	//for keeping threads from stomping on each other
	CRITICAL_SECTION m_USBCriticalSection;	//for keeping threads from stomping on each other
};

#endif // !defined(AFX_USBSOCKET_H__8B9CBE16_F075_4636_840E_8C00E4798110__INCLUDED_)
